unit UKeys;
{
	========================================================================

	Modulo:	UKeys	  	Tasko:	Gheneralaj klavaj funkcioj

    ========================================================================

    Funkcio:	Tiu chi modulo provizas gheneralajn metodojn por la analizo
    			de klavaj eventoj en la Vindoza eventstako. Tiuj unue
                primitive traktighas en la klaso TKey. La konstruilo ricevu
                la valorojn donatajn de Vindozo tra KeyProc(). Poste
                disponighas longa listo de buleajhoj kaj diversaj variantoj
                de la klaveventa enhavo.

                En la klaso TKeys trovighas bazaj metodoj por meti klavojn
                kun akompanaj senpashajhoj en la klaveventan stakon por tio,
                ke Vindozo transprenu kaj traktu ilin.

                Krom tio estas metodo por elekti specifan klavaron el la stako
                de klavaroj aktuale aktivigitaj de la uzanto. Evidente por
                tio, ke chio funkciu, estu inter ili internacia - alivorte
                diakritpova - Usona klavaro kaj klavaro el la Latin-3-a
                lingvogrupo. Apude havu la uzulo sian normalan nacilingvan
                klavaron. (Pri chio chi zorgas la Eka instalilo.)

    ========================================================================
    Dato		Autoro			Modifo
    ========================================================================
    99.01.01	Jury Finkel		Prapretigo
    99.08.01	Simon Payne		Reordigo kaj enkonduko de klasoj
    99.08.29	Simon Payne		Apartigo de speciala klavoklaso TKey
    99.08.29	Simon Payne		StripAltCtrlSend() aldonita
	99.09.04	Simon Payne		Send() virtualigita
    99.10.11	Simon Payne		Nun KeyDown() virtualigita anstatau Send()
    99.10.11	Simon Payne		Banto forigita el tSelectKeyboard() kaj cimo
    							solvita, se ghusta klavoro jam estas - ekz
    							jam internacia.
    ========================================================================
}

interface

uses
  Windows,
  SysUtils,
  Messages;

type

    TKey = class(TObject)
    private
        _bShift,
        _bLShift,
        _bRShift,
        _bCtrl,
        _bAlt,
        _bRAlt,
        _bCaps,
        _bDown,
        _bExt,
        _bAction,
        _bNoRemove: boolean;
        _iCode: integer;
        _lKey,
        _lParam: Longint;
        _cKey: char;
        _iScan: integer;
    public
        property bShift: boolean Read _bShift;
        property bLShift: boolean Read _bLShift;
        property bRShift: boolean Read _bRShift;
        property bCtrl: boolean Read _bCtrl;
        property bAlt: boolean Read _bAlt;
        property bRAlt: boolean Read _bRAlt;
        property bCaps: boolean Read _bCaps;
        property bDown: boolean Read _bDown;
        property bExt: boolean Read _bExt;
        property bAction: boolean Read _bAction;
        property bNoRemove: boolean Read _bNoRemove;
        property iCode: integer Read _iCode;
        property lKey: LongInt Read _lKey;
        property lParam: LongInt Read _lParam;
        property cKey: char Read _cKey;
        property iScan: integer Read _iScan;
        constructor Create(piCode: Integer; plKey, plParam: Longint);
    end;

    TKeys = class(TObject)
    private
       	_bNT: boolean;
    public
    	oKey: TKey;
        property bNT: boolean Read _bNT;
        constructor Create(pbNT:boolean);
        procedure KeyDown(piKey:integer); virtual;
        procedure KeyUp(piKey:integer);
        procedure Send(pcKey: Char);
        procedure AltCtrlSend(pcKey: Char);
        procedure StripAltCtrlSend(pcKey: Char);
        procedure SendBack;
        function bLatinKB: boolean;
        function tSelectKeyboard(ptKla:THandle):THandle;
end;

implementation

{===================================================================}

const

    DOWN_BIT = $80000000;
    EXT_BIT  = $20000000;
    KS_TOGGLE_ON = $01;
    KS_KEY_DOWN  = $80;

constructor TKey.Create(piCode: Integer; plKey, plParam: Longint);
begin
    _iCode:=piCode;
    _lKey:=plKey;
    _lParam:=plParam;
    _cKey:=chr(plKey);
    _iScan:=MapVirtualKey(_lKey,0);
    _bCaps:=GetKeyState(VK_CAPITAL)  and KS_TOGGLE_ON <> 0;
    _bShift:=GetKeyState(VK_SHIFT)   and KS_KEY_DOWN  <> 0;
    _bLShift:=GetKeyState(VK_LSHIFT) and KS_KEY_DOWN  <> 0;
    _bRShift:=GetKeyState(VK_RSHIFT) and KS_KEY_DOWN  <> 0;
    _bCtrl:=GetKeyState(VK_CONTROL)  and KS_KEY_DOWN  <> 0;
    _bAlt:=GetKeyState(VK_MENU)      and KS_KEY_DOWN  <> 0;
    _bRAlt:=GetKeyState(VK_RMENU)    and KS_KEY_DOWN  <> 0;
    _bDown:=(_lParam and DOWN_BIT)=0;
    _bExt:=(lKey and EXT_BIT>0);
    _bAction:=(iCode=HC_ACTION);
    _bNoRemove:=(iCode=HC_NOREMOVE);
end;

{===================================================================}

constructor TKeys.Create(pbNT: boolean);
begin
    _bNT:=pbNT;
end;

procedure TKeys.KeyDown(piKey: integer);
begin
    Keybd_Event(piKey,MapVirtualKey(piKey,0),0,0);
end;

procedure TKeys.KeyUp(piKey: integer);
begin
    Keybd_Event(piKey,MapVirtualKey(piKey,0),KEYEVENTF_KEYUP,0);
end;

procedure TKeys.Send(pcKey: Char);
begin
    KeyDown(ord(pcKey));
    KeyUp(ord(pcKey));
end;

procedure TKeys.AltCtrlSend(pcKey: Char);
begin
	if oKey=nil then
    	exit;
    with oKey do
    begin
		if not _bAlt then
    		KeyDown(VK_Menu);
    	if not _bCtrl then
    		KeyDown(VK_Control);
    	if _bCaps and _bNT then
    	begin
    		if not _bShift then
        		KeyDown(VK_Shift)
        	else
        		KeyUp(VK_Shift);
    	end;
    	Send(pcKey);
    	if _bCaps and _bNT then
    	begin
    		if not _bShift then
        		KeyUp(VK_Shift)
        	else
        		KeyDown(VK_Shift);
    	end;
    	if not _bCtrl then
    		KeyUp(VK_Control);
    	if not _bAlt then
    		KeyUp(VK_Menu);
    end;
end;

procedure TKeys.StripAltCtrlSend(pcKey: Char);
begin
	if oKey=nil then
    	exit;
    with oKey do
    begin
		if _bAlt then
    		KeyUp(VK_Menu);
    	if _bCtrl then
    		KeyUp(VK_Control);
    	if _bCaps and _bNT then
    	begin
    		if not _bShift then
        		KeyDown(VK_Shift)
        	else
        		KeyUp(VK_Shift);
    	end;
    	Send(pcKey);
    	if _bCaps and _bNT then
    	begin
    		if not _bShift then
        		KeyUp(VK_Shift)
        	else
        		KeyDown(VK_Shift);
    	end;
    	if _bCtrl then
    		KeyDown(VK_Control);
    	if _bAlt then
    		KeyDown(VK_Menu);
    end;
end;

procedure TKeys.SendBack;
begin
    Keybd_Event(VK_Back,MapVirtualKey(VK_Delete,0),0,0);
    Keybd_Event(VK_Back,MapVirtualKey(VK_Left,0),KEYEVENTF_KEYUP,0);
end;

function TKeys.bLatinKB: boolean;
begin
    Result:=VkKeyScan('u')>0;
end;

function TKeys.tSelectKeyboard(ptKla:THandle):THandle;
begin
	Result:=0;
    if ptKla=0 then
        exit;
    Result:=GetKeyboardLayout(0);
    if Result<>ptKla then
    	Result:=ActivateKeyboardLayout(ptKla,0);
end;

end.
